/*
 *
 *
 *    condredirbw.cc
 *
 *    Version 0.04
 *    
 *    Utility to test for presense (or absense) of email address in listfile
 *    and based on that, possibly run another command.
 *
 *    See README for details.
 *
 *    =========================================================================
 *
 *    Copyright (C) 2003 by Matt Roberts, KK5JY.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    =========================================================================
 *
 *
 */

#include <fstream>
#include <iostream>
#include <vector>

#include <unistd.h>

#include <my++/regex.h>
#include <my++/stype.h>

#define FOUND (0)
#define ERR_DEFER (111)


typedef std::vector<my::regex> bwlist_t;


bool matches(const std::string &s, const bwlist_t &l) {
	bwlist_t::const_iterator i;

	for (i = l.begin(); i != l.end(); ++i) {
		if ((*i) == s)
			return true;
	}

	return false;
}


int main (int argc, char **argv) {
	char *sender = NULL;
	char *config = NULL;
	bwlist_t whitelist;
	std::string s;
	bool invert = true;

	// make sure we have enough args
	if (argc < 3) {
		std::cerr << "usage: qmail-listfile <[-]list> <program> [<arg> ... ]" << std::endl;
		return ERR_DEFER;
	}

	// fetch the SENDER env-var
	sender = getenv("SENDER");
	if (!sender) {
		std::cerr << "SENDER environment variable not set" << std::endl;
		return ERR_DEFER;
	}

	// read the config file name
	config = argv[1];

	// deal with inverted list name
	if (*config == '-') {
		invert = ! invert;
		++config;
	}

	// if the length of the filename is zero, bail
	if (strlen(config) == 0) {
		std::cerr << "listfile name must be nonempty" << std::endl;
		return ERR_DEFER;
	}

	// open the listfile
	std::ifstream wl_file(config);
	if (!wl_file) {
		std::cerr << "could not read listfile: " << config << std::endl;
		return ERR_DEFER;
	}

	// load the regex array
	while (wl_file) {
		// get one line
		if (!std::getline(wl_file, s))
			continue;

		// trim the string
		s = my::rtrim(my::rtrim(s));

		// skip empty lines
		if (s.empty())
			continue;

		// also skip comments
		if ((s[0] == '#') || (s[0] == ';'))
			continue;

		// append the string
		whitelist.push_back(my::regex(s, my::regex::nosub | my::regex::icase));
	}

	// close the listfile
	wl_file.close();

	// search for sender (result depends on value of 'invert')
	bool m_result = matches(sender, whitelist);
	if (invert)
		m_result = ! m_result;
	if (m_result)
		return FOUND;
	
	// criteria not met - run program
	int result = execvp(argv[2], argv + 2);

	if (result < 0) {
		std::cerr << "Could not execvp() program: " << argv[2] << ": " << strerror(errno) << std::endl;
		return 111;
	}
}

/* EOF */
